﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;

namespace WebbyDraw
{
    public partial class ColorPicker : UserControl
    {
        private bool isMouseDownOnCanvas; 
        private double colorCanvasXProperty;
        private double colorCanvasYProperty;
        private SolidColorBrush baseHueProperty;
        private bool isCompactProperty;

        private SolidColorBrush selectedColorProperty;
        

        public ColorPicker()
        {
            InitializeComponent();

            colorCanvasXProperty = 1;
            ColorCanvasY = 1;

            isMouseDownOnCanvas = false;
            
        }

        #region Dependency Properties

        private double ColorCanvasX
        {
            get { return colorCanvasXProperty; }
            set
            {
                colorCanvasXProperty = value;
                UpdateSelectedColor();                
            }
        }

        private double ColorCanvasY
        {
            get { return colorCanvasYProperty; }
            set
            {
                colorCanvasYProperty = value;
                UpdateSelectedColor();
            }
        }

        private SolidColorBrush BaseHue
        {
            get { return baseHueProperty; }
            set
            {
                baseHueProperty = value;
                UpdateSelectedColor();
                if (BackgroundCanvas != null)
                {
                    BackgroundCanvas.Background = baseHueProperty;
                }
            }
        }

        public bool IsCompact
        {
            get { return isCompactProperty; }
            set 
            { 
                isCompactProperty = value;
                if (LayoutRoot != null)
                {
                    CompactLayoutChange(value);
                }
            }
        }

        private void CompactLayoutChange(bool compactChange)
        {
            if (compactChange)
            {
                LargePanel.Visibility = Visibility.Collapsed;
                CompactPanel.Visibility = Visibility.Visible;
                rightColumn.MinWidth = 0;
                RootControl.Width = 130;
                RootControl.Height = 200;
            }
            else
            {
                LargePanel.Visibility = Visibility.Visible;
                CompactPanel.Visibility = Visibility.Collapsed;
                rightColumn.MinWidth = 90;
                RootControl.Height = double.NaN;
                RootControl.Width = double.NaN;
            }
        }

        public SolidColorBrush SelectedColor
        {
            get { return selectedColorProperty; }
            set 
            { 
                selectedColorProperty = value;

                if (resultCanvas != null)
                {
                    resultCanvas.Background = selectedColorProperty;
                }
            }
        }

#endregion




        private void UpdateSelectedColor()
        {
            Color baseColor = ((System.Windows.Media.SolidColorBrush)(baseHueProperty)).Color;

            Color newColor = new Color();
            if (colorCanvasXProperty > 1.0)
            {
                colorCanvasXProperty = 1.0;
            }

            if (colorCanvasYProperty > 1.0)
            {
                colorCanvasYProperty = 1.0;
            }

            newColor.R = System.Convert.ToByte(colorCanvasYProperty * (baseColor.R + ((255 - baseColor.R) * colorCanvasXProperty)));
            newColor.G = System.Convert.ToByte(colorCanvasYProperty * (baseColor.G + ((255 - baseColor.G) * colorCanvasXProperty)));
            newColor.B = System.Convert.ToByte(colorCanvasYProperty * (baseColor.B + ((255 - baseColor.B) * colorCanvasXProperty)));

            newColor.A = 255;

            SolidColorBrush updatedColor = new SolidColorBrush(newColor);

            SelectedColor = new SolidColorBrush(newColor);

            if (CopyColorText != null)
            {
                RedText.Text = newColor.R.ToString();
                GreenText.Text = newColor.G.ToString();
                BlueText.Text = newColor.B.ToString();
                CopyColorText.Text = CompactRGBText.Text = "" + newColor.R.ToString() + "," + newColor.G.ToString() + "," + newColor.B.ToString();
                CopyHexText.Text = CompactHexText.Text = newColor.ToString();
            }              
        }

        #region Color Calculations

        private byte GetRedValue(double range, double value)
        {
            double percentage = value / range;
            int redValue = 0;

            if (percentage < .3333)
            {
                if(percentage < .16666)
                {
                    redValue=255;
                } 
                else
                {
                    redValue = System.Convert.ToInt32(255 * (Math.Abs(.3333 - percentage) / .16666));
                }
            }
            else if (percentage > .66666)
            {
                if(percentage > .83333)
                {
                    redValue=255;
                } 
                else
                {
                    redValue = System.Convert.ToInt32(255 * (Math.Abs(percentage - .6666) / .16666));
                }
            }

            return System.Convert.ToByte(redValue);
        }

        private byte GetBlueValue(double range, double value)
        {
            double percentage = value / range;
            int blueValue = 0;

            if(percentage > .66666){
                blueValue = 0;
            }
            else if (percentage < .16666)
            {
                blueValue = System.Convert.ToInt32(255 * (Math.Abs(percentage) / .16666));
            }
            else if (percentage < .5)
            {
                blueValue = 255;
            } 
            else if (percentage > .5)
            {
                blueValue = System.Convert.ToInt32(255 * (Math.Abs(.66666 - percentage) / .16666));
            }

            return System.Convert.ToByte(blueValue);
        }

        private byte GetGreenValue(double range, double value)
        {
            double percentage = value / range;
            int greenValue = 0;

            if (percentage < .3333)
            {
                greenValue = 0;
            }
            else if(percentage > .8333)
            {
                greenValue = System.Convert.ToInt32(255 * (Math.Abs(1 - percentage) / .16666));
            }
            else if(percentage > .5)
            {
                greenValue= 255;
            }
            else if(percentage < .5)
            {
                greenValue = System.Convert.ToInt32(255 * (Math.Abs(.3333 - percentage) / .16666));
            }
            
            return System.Convert.ToByte(greenValue);
        }

        #endregion

        private void UpdateColorCanvas(object sender, RoutedEventArgs e)
        {
            Slider thisSlider = sender as Slider;

            Color targetColor = new Color();
           
            double range = thisSlider.Maximum - thisSlider.Minimum;

            targetColor.R = GetRedValue(range, thisSlider.Value);
            targetColor.G = GetGreenValue(range, thisSlider.Value);
            targetColor.B = GetBlueValue(range, thisSlider.Value);
            targetColor.A = 255;

            BaseHue = new SolidColorBrush(targetColor);
        }

        #region Color Canvas Mouse Event Handlers

        private void Canvas_MouseLeftButtonUp(object sender, MouseButtonEventArgs e)
        {
            isMouseDownOnCanvas = false;
        }

        private void Canvas_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            //Let the dragging issue begin
            isMouseDownOnCanvas = true;
            Canvas backgroundCanvas = sender as Canvas;

            //move the little circular canvas thingy
            Point newGridPoint = e.GetPosition(backgroundCanvas);
            Canvas.SetTop(this.FinalColor, (newGridPoint.Y - 6));
            Canvas.SetLeft(this.FinalColor, (newGridPoint.X - 6));
                       
            //Set the new Brush
            colorCanvasXProperty = (Math.Abs(backgroundCanvas.ActualWidth - newGridPoint.X))/backgroundCanvas.ActualWidth;
            ColorCanvasY = (Math.Abs(newGridPoint.Y - backgroundCanvas.ActualHeight)) / backgroundCanvas.ActualHeight;
        }

        private void Canvas_MouseMove(object sender, MouseEventArgs e)
        {
            if (isMouseDownOnCanvas)
            {
                Canvas backgroundCanvas = sender as Canvas;

                Point newGridPoint = e.GetPosition(backgroundCanvas);
                Canvas.SetTop(this.FinalColor, (newGridPoint.Y - 6));
                Canvas.SetLeft(this.FinalColor, (newGridPoint.X - 6));

                //Set the new Brush
                colorCanvasXProperty = (Math.Abs(backgroundCanvas.ActualWidth - newGridPoint.X)) / backgroundCanvas.ActualWidth;
                ColorCanvasY = (Math.Abs(newGridPoint.Y - backgroundCanvas.ActualHeight)) / backgroundCanvas.ActualHeight;
            }
        }
        
        private void Canvas_MouseLeave(object sender, MouseEventArgs e)
        {
            //find out which side the mouse left
            if (isMouseDownOnCanvas)
            {
                if ((colorCanvasXProperty - 1) > -.1)
                {
                    colorCanvasXProperty = 1;
                }
                else if ((colorCanvasXProperty - .1) < 0)
                {
                    colorCanvasXProperty = 0;
                }
                
                if ((colorCanvasYProperty - 1) > -.1)
                {
                    colorCanvasYProperty = 1;
                }
                else if ((colorCanvasYProperty - .1) < 0)
                {
                    colorCanvasYProperty = 0;
                }

                ColorCanvasY = colorCanvasYProperty;
            }
        }

        #endregion
    }

    public class ColorChangedEventArgs : EventArgs
    {
        public ColorChangedEventArgs(SolidColorBrush oldColor, SolidColorBrush newColor)
        {
            this.oldColor = oldColor;
            this.newColor = newColor;
        }

        // The fire event will have two pieces of information--
        // 1) Where the fire is, and 2) how "ferocious" it is.

        public SolidColorBrush oldColor;
        public SolidColorBrush newColor;

    }



}
